import datetime
import logging
from pygeodesy.ellipsoidalVincenty import LatLon

MIN_SPEED = 5
MAX_SPEED = 40
MAX_LATENCY = 5

logger = logging.getLogger(__name__)


class StatumRunner:

    def __init__(self, market, race_change):
        self._market = market
        self.race_change = race_change
        self._active = True
        self._removed = False

        self.position = LatLon(race_change.lat, race_change.long)
        self.runner_distance_to_finish = None
        self.runner_seconds_to_finish = None

    def __call__(self, race_change, runner_book):
        if isinstance(runner_book, dict):
            status = runner_book['status']
        else:
            status = runner_book.status
        if status != 'ACTIVE':
            self._removed = True
        else:
            self._removed = False  # just in case there is an accidental removal
        self.race_change = race_change
        self.position = LatLon(race_change.lat, race_change.long)

        self.runner_distance_to_finish = self._market.runner_distance_to_finish(self.selection_id)
        self.runner_seconds_to_finish = self._market.runner_seconds_to_finish(
            self.selection_id, distance=self.runner_distance_to_finish
        )

    def check_active(self):
        """ Check that speed looks valid.
        """
        if self.race_change and MIN_SPEED < self.race_change.speed < MAX_SPEED:
            self._active = True
        else:
            self._active = False

    @property
    def progress(self):  # added 09/11/2018
        return self.race_change.progress

    @property
    def progress_seconds_to_finish(self):
        if self.progress and self.speed:
            return self.progress / self.speed

    @property
    def speed(self):
        return self.race_change.speed

    @property
    def stride_frequency(self):  # added 09/11/2018
        return self.race_change.stride_frequency

    @property
    def active(self):
        if self._active and not self._removed:
            return True
        else:
            return False

    @property
    def feed_latency(self):
        if self.race_change:
            return (datetime.datetime.utcnow() - self.race_change.feed_time).total_seconds()
        else:
            return 0

    @property
    def pred_seconds_from_leader(self):
        return self.runner_seconds_to_finish - self._market.leader_seconds_to_finish

    @property
    def selection_id(self):
        return self.race_change.selection_id

    def __str__(self):
        return 'StatumRunner {0} {1}'.format(self.selection_id, self.active)
