import unittest
import datetime
from unittest import mock

from statum.runner import StatumRunner


class TestStatumRunner(unittest.TestCase):

    def setUp(self):
        self.race_change = mock.Mock()
        self.race_change.selection_id = 4312
        self.race_change.lat = 12
        self.race_change.long = -12
        self.race_change.speed = 7
        self.mock_market = mock.Mock()
        self.runner = StatumRunner(self.mock_market, self.race_change)

    def test_init(self):
        self.assertEqual( self.runner._market, self.mock_market)
        self.assertEqual( self.runner.race_change, self.race_change)
        self.assertTrue(self.runner._active)
        self.assertFalse(self.runner._removed)

        self.assertIsNotNone(self.runner.position)
        self.assertIsNone(self.runner.runner_distance_to_finish)
        self.assertIsNone(self.runner.runner_seconds_to_finish)

    def test_check_active(self):
        self.runner.check_active()
        self.assertTrue(self.runner._active)

        self.race_change.speed = 4
        self.runner.check_active()
        self.assertFalse(self.runner._active)

        self.race_change.speed = 23
        self.runner.check_active()
        self.assertTrue(self.runner._active)

        self.race_change.speed = 41
        self.runner.check_active()
        self.assertFalse(self.runner._active)

        self.race_change.speed = 25
        self.runner.check_active()
        self.assertTrue(self.runner._active)

    def test_progress(self):
        self.race_change.progress = 2048
        self.assertEqual(self.runner.progress, 2048)
        self.race_change.progress = None
        self.assertIsNone(self.runner.progress)

    def test_progress_seconds_to_finish(self):
        self.race_change.progress = 2048
        self.race_change.speed = 25.6
        self.assertEqual(self.runner.progress_seconds_to_finish, 80.0)

    def test_speed(self):
        self.race_change.speed = 25.6
        self.assertEqual(self.runner.speed, 25.6)
        self.race_change.speed = None
        self.assertIsNone(self.runner.speed)

    def test_stride_frequency(self):
        self.race_change.stride_frequency = 1.28
        self.assertEqual(self.runner.stride_frequency, 1.28)
        self.race_change.stride_frequency = None
        self.assertIsNone(self.runner.stride_frequency)

    def test_active(self):
        self.assertTrue(self.runner.active)
        self.runner._active = False
        self.assertFalse(self.runner.active)
        self.runner._active = False
        self.runner._removed = True
        self.assertFalse(self.runner.active)

    def test_feed_latency(self):
        self.race_change.feed_time = datetime.datetime.utcnow()
        self.assertGreater(self.runner.feed_latency, 0)

    def test_pred_seconds_from_leader(self):
        self.runner.runner_seconds_to_finish = 11.9
        self.mock_market.leader_seconds_to_finish = 12.0

        self.assertEqual(self.runner.pred_seconds_from_leader, -0.09999999999999964)

    def test_selection_id(self):
        self.assertEqual(self.runner.selection_id, 4312)

    def test_str(self):
        self.assertEqual(str(self.runner), 'StatumRunner 4312 True')
